package app.main;

import ryulib.game.GameControl;
import ryulib.game.GameControlGroup;
import ryulib.game.GamePlatformInfo;
import android.graphics.Canvas;
import android.graphics.Paint;
import app.resources.Resource;
import app.resources.ResourceShip;

public class Ship extends GameControl {

	private static final int _ANIMATIONINTERVAL = 100;
	private static final int _SPEED = 150;
	
	public Ship(GameControlGroup gameControlGroup) {
		super(gameControlGroup);
	
	}

	private Canvas _Canvas = null;
	private Paint _Paint = null;
	
	private JoyStick _JoyStick = new JoyStick(_SPEED);
	
	private AnimationCounter _AnimationCounter = 
		new AnimationCounter(
				_ANIMATIONINTERVAL, 
				ResourceShip.getInstance().getCount()
		);
	
	@Override
	protected void onStart(GamePlatformInfo platformInfo) {
		_Canvas = platformInfo.getCanvas();
		_Paint = platformInfo.getPaint();
		
		_JoyStick.PrepareOrientationSensor(Resource.getInstance().getContext());

		_JoyStick.setBoundaryLimit(
				true, 
				0, 0, _Canvas.getWidth(), _Canvas.getHeight(),
				ResourceShip.WIDTH, ResourceShip.HEIGHT
		);
	}
	
	@Override
	protected void onTick(GamePlatformInfo platformInfo) {
		long tick = platformInfo.getTick();
		
		_AnimationCounter.Tick(tick);
		_JoyStick.Tick(tick);
	}
	
	@Override
	protected void onDraw(GamePlatformInfo platformInfo) {
		_Canvas.drawBitmap(
				ResourceShip.getInstance().getBitmap(_AnimationCounter.getIndex()), 
				_JoyStick.getX(), _JoyStick.getY(), 
				_Paint
		);
	}
	
}
