package app.main;

import ryulib.game.GameControl;
import ryulib.game.GameControlGroup;
import ryulib.game.GamePlatformInfo;
import ryulib.game.HitArea;
import ryulib.graphic.Boundary;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.view.KeyEvent;
import android.view.MotionEvent;
import app.resources.Resource;
import app.resources.ResourceShip;

public class Ship extends GameControl {

	private static final int _ANIMATIONINTERVAL = 100;
	private static final int _SPEED = 150;
	private static final int _HORIZONTAL_MARGINE = 20;
	private static final int _VERTICAL_MARGINE   = 15;
	
	public Ship(GameControlGroup gameControlGroup) {
		super(gameControlGroup);
	
		_HitArea.Add(_HitBoundary);
	}

	private Canvas _Canvas = null;
	private Paint _Paint = null;
	
	private JoyStick _JoyStick = new JoyStick(_SPEED);
	
	private AnimationCounter _AnimationCounter = 
		new AnimationCounter(
				_ANIMATIONINTERVAL, 
				ResourceShip.getInstance().getCount()
		);
	
	// 浹 ˻   ڽ ϰ ִ  ˷ش.  
	private HitArea _HitArea = new HitArea();

	// Boundary Rect   Ѵ.  
	private Boundary _HitBoundary = new Boundary(0, 0, 0, 0);
	
	@Override
	protected HitArea getHitArea() {
		//  ڸ 浹 Ͼ ʵ  ְ ִ.
		_HitBoundary.setBoundary(
				_HORIZONTAL_MARGINE, 
				_JoyStick.getX() + _VERTICAL_MARGINE, 
				ResourceShip.WIDTH - _HORIZONTAL_MARGINE*2, 
				_JoyStick.getY() + ResourceShip.HEIGHT - _VERTICAL_MARGINE*2
		);
		
		return _HitArea;
	}
	
	@Override
	protected void onStart(GamePlatformInfo platformInfo) {
		_Canvas = platformInfo.getCanvas();
		_Paint = platformInfo.getPaint();
		
		_JoyStick.PrepareOrientationSensor(Resource.getInstance().getContext());

		_JoyStick.setBoundaryLimit(
				true, 
				0, 0, _Canvas.getWidth(), _Canvas.getHeight(),
				ResourceShip.WIDTH, ResourceShip.HEIGHT
		);
	}
	
	@Override
	protected void onTick(GamePlatformInfo platformInfo) {
		long tick = platformInfo.getTick();
		
		_AnimationCounter.Tick(tick);
		_JoyStick.Tick(tick);
	}
	
	@Override
	protected void onDraw(GamePlatformInfo platformInfo) {
		_Canvas.drawBitmap(
				ResourceShip.getInstance().getBitmap(_AnimationCounter.getIndex()), 
				_JoyStick.getX(), _JoyStick.getY(), 
				_Paint
		);
		
		GameControl GameControl = this.CheckCollision(this);
		if ((GameControl != null) && (GameControl instanceof Asteroid)) {
			GameControl.Delete();
			this.Delete();
		}
	}
	
	@Override
	protected boolean onKeyDown(GamePlatformInfo platformInfo, int keyCode, KeyEvent msg) {
		_JoyStick.onKeyDown(platformInfo, keyCode, msg);
		return true;
    }

	@Override
    protected boolean onKeyUp(GamePlatformInfo platformInfo, int keyCode, KeyEvent msg) {
		_JoyStick.onKeyUp(platformInfo, keyCode, msg);
		return true;
    }

	@Override
    protected boolean onTouchEvent(GamePlatformInfo platformInfo, MotionEvent event) {
		int x = (int) event.getX();

		if (x <= ResourceShip.WIDTH) {
			_JoyStick.onTouchEvent(platformInfo, event);
		} else {
			// TODO Fire();
		}    
		return true;
    }
    
}
