package app.main;

import ryulib.game.GameControl;
import ryulib.game.GamePlatformInfo;
import ryulib.game.HitArea;
import ryulib.graphic.Boundary;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import app.resources.BitmapList;
import app.resources.ResourceAsteroid;
import app.resources.ResourceExplodingAsteroid;

public class Asteroid extends GameControl {

	private static final int _ANIMATIONINTERVAL = 100;
	private static final int _SPEED = 100;
	private static final int _HORIZONTAL_MARGINE = 5;
	private static final int _VERTICAL_MARGINE   = 5;

	public Asteroid(int x, int y) {
		super(null);
	
		_X = x;
		_Y = y;
		
		_HitArea.Add(_HitBoundary);
	}

	private Canvas _Canvas = null;
	private Paint _Paint = null;
	
	private int _X = 0;
	private int _Y = 0;
	private Scroll _Scroll = new Scroll(_SPEED);
	
	private BitmapList _BitmapList = null;	
	private AnimationCounter _AnimationCounter = 
		new AnimationCounter(
				_ANIMATIONINTERVAL, 
				ResourceAsteroid.getInstance().getCount()
		);
	
	// 浹 ˻   ڽ ϰ ִ  ˷ش.  
	private HitArea _HitArea = new HitArea();

	// Boundary Rect   Ѵ.  
	private Boundary _HitBoundary = new Boundary(0, 0, 0, 0);
	
	@Override
	protected HitArea getHitArea() {
		if (isExploding()) return null;
		
		_HitBoundary.setBoundary(
				_X + _HORIZONTAL_MARGINE, 
				_Y + _VERTICAL_MARGINE, 
				_X + ResourceAsteroid.SIZE - _HORIZONTAL_MARGINE*2, 
				_Y + ResourceAsteroid.SIZE - _VERTICAL_MARGINE*2
		);		
		
		return _HitArea;
	}
	
	public void Explode() {
		_AnimationCounter.Clear();
		_AnimationCounter.setSize(ResourceExplodingAsteroid.getInstance().getCount());
		_AnimationCounter.setAutoRewind(false);

		_BitmapList = ResourceExplodingAsteroid.getInstance();
	}
	
	@Override
	protected void onStart(GamePlatformInfo platformInfo) {
		_Canvas = platformInfo.getCanvas();
		_Paint = platformInfo.getPaint();
		
		_BitmapList = ResourceAsteroid.getInstance();
	}
	
	@Override
	protected void onTick(GamePlatformInfo platformInfo) {
		long tick = platformInfo.getTick();
		
		_AnimationCounter.Tick(tick);
		
		if (isExploding() == false) {
			// ʴ _Scroll.getSpeed() ŭ ӵ  ̵ Ѵ.
			_X = _X - _Scroll.Move(tick);

			// ȭ鿡   Ѵ.
			if (_X < (-ResourceAsteroid.SIZE)) this.Delete(); 
		}
	}
	
	@Override
	protected void onDraw(GamePlatformInfo platformInfo) {
		Bitmap bitmap = _BitmapList.getBitmap(_AnimationCounter.getIndex());
		
		if (bitmap != null) {
			_Canvas.drawBitmap(bitmap, _X, _Y, _Paint);
		} else {
			this.Delete();
		}
	}
	
	public boolean isExploding() {
		return _BitmapList == ResourceExplodingAsteroid.getInstance();
	}
	
}
