package ryulib.game;

import ryulib.OnHandlerMessageListener;
import ryulib.OnNotifyEventListener;
import ryulib.ThreadRepeater;
import android.content.Context;
import android.graphics.Canvas;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class GamePlatform extends SurfaceView implements SurfaceHolder.Callback {

	public GamePlatform(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		
		init();
	}

	public GamePlatform(Context context, AttributeSet attrs) {
		super(context, attrs);

		init();
	}

	public GamePlatform(Context context) {
		super(context);

		init();
	}

	private void init() {
	    _SurfaceHolder.addCallback(this);
		_Repeater.setOnNotifyEvent(_OnRepeat);
		
	    // Ű ̺Ʈ ޱ ؼ Ŀ Ȯ
	    setFocusable(true);
	}	

	private GameControlGroup _GameControlGroup = new GameControlGroup(null);
	private GamePlatformInfo _PlatformInfo = new GamePlatformInfo(this);
    private SurfaceHolder _SurfaceHolder = getHolder();
	private ThreadRepeater _Repeater = new ThreadRepeater();

	// Property
	private boolean _UseKeyEvent = false;
	private boolean _UseMotionEvent = false;
	private boolean _AutoInvalidate = false;  

	public void Start() {
		_OldTick = System.currentTimeMillis();
		_Repeater.Start();
	}
	
	public void Stop() {
		_Repeater.Stop();
	}
	
	public void Invalidate() {
		_OnRepeat.onNotify(_Repeater);
	}
	
	public void ClearControls() {
		_GameControlGroup.ClearControls();
	}
	
	public void AddControl(GameControlBase control) {
		_GameControlGroup.AddControl(control);
	}

	public void RemoveControl(GameControlBase control) {
		_GameControlGroup.RemoveControl(control);
	}
	
	public void surfaceCreated(SurfaceHolder holder) {
		Start();
	}

	public void surfaceDestroyed(SurfaceHolder holder) {
		Stop();
	}
	
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
    	_PlatformInfo.setSize(width, height);
    	// TODO : GameControlBase OnSurfaceChanged ߰
	}

	private long _OldTick = System.currentTimeMillis();

    private OnNotifyEventListener _OnRepeat = new OnNotifyEventListener() {
    	@Override
		public synchronized void onNotify(Object sender) {
			Canvas canvas = _SurfaceHolder.lockCanvas(null);
            try {
                if (canvas == null) return;
	                
				long tick = System.currentTimeMillis();
				_PlatformInfo.setTick(tick - _OldTick);
				
				_PlatformInfo._GameMessage = _PlatformInfo._GameMessageList.Get();

				_GameControlGroup.onRepeate(_PlatformInfo);

				_OldTick = tick;

        		_PlatformInfo.Draw(canvas);
            } finally {
                if (canvas != null) {
                	_SurfaceHolder.unlockCanvasAndPost(canvas);
                }
	    	}
            
            _GameControlGroup.ArrangeControls();
		}
	};
	
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent msg) {
    	if (_UseKeyEvent) {
    		_GameControlGroup.onKeyDown(_PlatformInfo, keyCode, msg);
    		if (_AutoInvalidate) Invalidate();
    	}
        return true;
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent msg) {
    	if (_UseKeyEvent) {
    		_GameControlGroup.onKeyUp(_PlatformInfo, keyCode, msg);
    		if (_AutoInvalidate) Invalidate();
    	}
        return true;
    }

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		if (_UseMotionEvent) {
			_GameControlGroup.onTouchEvent(_PlatformInfo, event);
    		if (_AutoInvalidate) Invalidate();
		}
		return true;
	}
	
	public void setOnHandlerMessage(OnHandlerMessageListener value) {
		_PlatformInfo.setOnHandlerMessage(value);
	}

	public void setUseKeyEvent(boolean _UseKeyEvent) {
		this._UseKeyEvent = _UseKeyEvent;
	}

	public boolean getUseKeyEvent() {
		return _UseKeyEvent;
	}

	public void setUseMotionEvent(boolean _UseMotionEvent) {
		this._UseMotionEvent = _UseMotionEvent;
	}

	public boolean getUseMotionEvent() {
		return _UseMotionEvent;
	}

	public void setRepeatInterval(long value) {
		_Repeater.setInterval(value);
	}

	public long getRepeatInterval() {
		return _Repeater.getInterval();
	}

	public void setAutoInvalidate(boolean value) {
		// ThreadRepeater  ϰ,  Ű ޽  ó  ȭ  Ѵ.
		_AutoInvalidate = value;
	}

	public boolean isAutoInvalidate() {
		return _AutoInvalidate;
	}

}
